// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const GLib = imports.gi.GLib;
const Shell = imports.gi.Shell;
const St = imports.gi.St;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const Gettext = imports.gettext;
const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

const DEFAULT_BASIC_SETTINGS = [
    [ _("System Settings"), 'gnome-control-center', 'preferences-system' ],
    [ _("GNOME Shell Extensions"), 'gnome-shell-extension-prefs', 'preferences-other' ],
    [ _("Startup Applications"), 'gnome-session-properties', 'session-properties' ],
    [ _("GNOME Tweak Tool"), 'gnome-tweak-tool', 'preferences-system' ],
    [ _("Synaptic Package Manager"), 'synaptic-pkexec', 'synaptic' ],
    [ _("Configure printers"), 'system-config-printer', 'printer' ],
    [ _("Users Settings"), 'users-admin', 'config-users' ]
];

function _getAggregateMenu() {
    try { return Main.panel.statusArea.aggregateMenu; } catch(e) { return null; }
}

const GNOME_AGGREGATEMENU = _getAggregateMenu();

const KEY_SHOW_STATUSAREA = 'show-statusarea';
const KEY_SHOW_STATUSMENU = 'show-statusmenu';

const STATUSAREA_DEFAULT = 0;

const STATUSMENU_DEFAULT = 0;
const STATUSMENU_ICON = 1;

const PopupBasicSettingsMenuSection = new Lang.Class({
    Name: 'PopupBasicSettingsMenuSection',
    Extends: PopupMenu.PopupMenuSection,

    _init: function() {
        this.parent();
        this._appSystem = Shell.AppSystem.get_default();
        this._connectAppSystem = this._appSystem.connect('installed-changed', Lang.bind(this, this._onInstalledChanged));
        this._onInstalledChanged();
        this.actor.connect('destroy', Lang.bind(this, function() {
            this._appSystem.disconnect(this._connectAppSystem);
        }));
    },

    _onInstalledChanged: function() {
        this.removeAll();
        let apps = [];
        for each(let app in DEFAULT_BASIC_SETTINGS) {
            if (GLib.find_program_in_path(app[1])) apps.push(app);
        }
        apps.sort(function(a, b) {
            return Convenience.getSorted(a[0], b[0]);
        });
        for (let i = 0; i < apps.length; i++) {
            let app = apps[i];
            let item = new PopupMenu.PopupMenuItem(app[0]);
            item.connect('activate', Lang.bind(this, function() {
                Main.overview.hide();
                Main.Util.spawnCommandLine(app[1]);
            }));
            this.addMenuItem(item);
        }
    }
});

const BasicSettingsIndicator = new Lang.Class({
    Name: 'BasicSettingsIndicator',
    Extends: PanelMenu.SystemIndicator,

    _init: function() {
        this.parent();
        this._indicator = this._addIndicator();
        this._indicator.icon_name = 'preferences-system-symbolic';
        this._item = new PopupMenu.PopupSubMenuMenuItem(_("Basic Settings"), true);
        this._item.icon.icon_name = 'preferences-system-symbolic';
        this._item.menu.addMenuItem(new PopupBasicSettingsMenuSection);
        this.menu.addMenuItem(this._item);
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_STATUSAREA, Lang.bind(this, this._updateVisibility));
        this._updateVisibility();
    },

    _updateVisibility: function() {
        let visible = this._settings.get_enum(KEY_SHOW_STATUSAREA) != STATUSAREA_DEFAULT && this._item.menu.numMenuItems > 0;
        this.indicators.visible = visible;
        this.menu.actor.visible = visible;
    },

    destroy: function() {
        this._settings.disconnect(this._connectSettings);
        this.menu.destroy();
        this._indicator.destroy();
        this.indicators.destroy();
    }
});

const AggregateMenu = new Lang.Class({
    Name: 'AggregateMenu',

    _init: function() {
        this._basicSettings = new BasicSettingsIndicator();
        this._indicators = new St.BoxLayout({ style_class: 'panel-status-indicators-box' });
        let children = GNOME_AGGREGATEMENU._indicators.get_children();
        for (let i = 0; i < children.length - 1; i++) {
            GNOME_AGGREGATEMENU._indicators.remove_actor(children[i]);
            this._indicators.add_child(children[i]);
        }
        this._indicators.add_child(this._basicSettings.indicators);
        this._iconPower = new St.Icon({ icon_name: 'system-shutdown-symbolic', style_class: 'system-status-icon' });
        GNOME_AGGREGATEMENU._indicators.insert_child_at_index(this._iconPower, 0);
        GNOME_AGGREGATEMENU._indicators.insert_child_at_index(this._indicators, 0);
        let menuItems = GNOME_AGGREGATEMENU.menu._getMenuItems();
        for (let i = menuItems.length - 1; i >= 0; i--) {
            if (menuItems[i] == GNOME_AGGREGATEMENU._system.menu) GNOME_AGGREGATEMENU.menu.addMenuItem(this._basicSettings.menu, i);
        }
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_STATUSMENU, Lang.bind(this, this._onShowStatusMenu));
        this._onShowStatusMenu();
    },

    _onShowStatusMenu: function() {
        this._updateVisibility(this._settings.get_enum(KEY_SHOW_STATUSMENU));
    },

    _updateVisibility: function(showStatusMenu) {
        this._indicators.visible = showStatusMenu != STATUSMENU_ICON;
        //this._iconPower.visible = showStatusMenu == STATUSMENU_ICON;
    },

    destroy: function() {
        this._settings.disconnect(this._connectSettings);
        this._updateVisibility(STATUSMENU_DEFAULT);
        GNOME_AGGREGATEMENU._indicators.remove_actor(this._iconPower);
        GNOME_AGGREGATEMENU._indicators.remove_actor(this._indicators);
        let children = this._indicators.get_children();
        for (let i = children.length - 1; i >= 0; i--) {
            this._indicators.remove_actor(children[i]);
            GNOME_AGGREGATEMENU._indicators.insert_child_at_index(children[i], 0);
        }
        this._iconPower.destroy();
        this._basicSettings.destroy();
    }
});

function getExtension() {
    return GNOME_AGGREGATEMENU ? new AggregateMenu() : null;
}
