// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const Gio = imports.gi.Gio;
const UPower = imports.gi.UPowerGlib;

const Main = imports.ui.main;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const Gettext = imports.gettext;
const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

function _getBattery() {
    if (Convenience.checkGSV(3, 4)) {
        try { return Main.panel._statusArea.battery; } catch(e) { return null; }
    } else  if (Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
        try { return Main.panel.statusArea.battery; } catch(e) { return null; }
    } else {
        try { return Main.panel.statusArea.aggregateMenu._power; } catch(e) { return null; }
    }
}

const GNOME_BATTERY = _getBattery();

const KEY_SHOW_BATTERY = 'show-battery';

const PANELMENU_BUTTON_DEFAULT = 0;
const PANELMENU_BUTTON_SHOW = 1;

let _prototypeDevicesChanged;
let _prototypeSync;

function _overrideFunction(enabled) {
    if (enabled) {
        if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
            _prototypeDevicesChanged = imports.ui.status.power.Indicator.prototype._devicesChanged;
            GNOME_BATTERY._devicesChanged = function() {
                _prototypeDevicesChanged.call(this);
                let pmShowBattery = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA).get_enum(KEY_SHOW_BATTERY);
                if (pmShowBattery != PANELMENU_BUTTON_DEFAULT) this.actor.visible = pmShowBattery == PANELMENU_BUTTON_SHOW;
                if (this.mainIcon) this.mainIcon.visible = this.actor.visible;
            };
        } else {
            _prototypeSync = imports.ui.status.power.Indicator.prototype._sync;
            GNOME_BATTERY._sync = function() {
                if (Convenience.checkGSV(3, 10)) {
                    function isBattery(result) {
                        if (!result) return false;
                        let [device] = result;
                        let [, deviceType] = device;
                        return (deviceType == UPower.DeviceKind.BATTERY);
                    }
                    this._proxy.GetPrimaryDeviceRemote(Lang.bind(this, function(result, error) {
                        let visible = false;
                        if (isBattery(result)) {
                            let [device] = result;
                            let [,, icon] = device;
                            let gicon = Gio.icon_new_for_string(icon);
                            this._indicator.gicon = gicon;
                            this._item.icon.gicon = gicon;
                            this._item.status.text = this._statusForDevice(device);
                            visible = true;
                        } else {
                            this._indicator.icon_name = 'battery-missing-symbolic';
                            this._item.icon.icon_name = 'battery-missing-symbolic';
                            this._item.status.text = _("Off");
                        }
                        let pmShowBattery = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA).get_enum(KEY_SHOW_BATTERY);
                        if (pmShowBattery != PANELMENU_BUTTON_DEFAULT) visible = pmShowBattery == PANELMENU_BUTTON_SHOW;
                        this._item.actor.visible = visible;
                        this._indicator.visible = visible;
                    }));
                } else if (Convenience.checkGSV(3, 12)) {
                    let visible = this._proxy.IsPresent;
                    if (visible) {
                        let icon = this._proxy.IconName;
                        this._indicator.icon_name = icon;
                        this._item.icon.icon_name = icon;
                        this._item.label.text = this._proxy.Type == UPower.DeviceKind.UPS ? _("UPS") : _("Battery");
                        this._item.status.text = this._getStatus();
                    } else {
                        this._indicator.icon_name = 'battery-missing-symbolic';
                        this._item.icon.icon_name = 'battery-missing-symbolic';
                        this._item.label.text = _("Battery");
                        this._item.status.text = _("Not Connected");
                    }
                    let pmShowBattery = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA).get_enum(KEY_SHOW_BATTERY);
                    if (pmShowBattery != PANELMENU_BUTTON_DEFAULT) visible = pmShowBattery == PANELMENU_BUTTON_SHOW;
                    this._item.actor.visible = visible;
                    this._indicator.visible = visible;
                } else {
                    _prototypeSync.call(this);
                    if (!this._proxy.IsPresent) {
                        this._indicator.icon_name = 'battery-missing-symbolic';
                        this._item.icon.icon_name = 'battery-missing-symbolic';
                        if (Convenience.checkGSV(3, 14) || Convenience.checkGSV(3, 16)) {
                            this._item.label.text = _("Battery");
                            this._item.status.text = _("Not Connected");
                        } else {
                            this._item.label.text = _("Not In Use");
                        }
                    }
                    let pmShowBattery = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA).get_enum(KEY_SHOW_BATTERY);
                    if (pmShowBattery != PANELMENU_BUTTON_DEFAULT) this._item.actor.visible = pmShowBattery == PANELMENU_BUTTON_SHOW;
                    this._indicator.visible = this._item.actor.visible;
                }
            };
        }
    } else {
        if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
            GNOME_BATTERY._devicesChanged = _prototypeDevicesChanged;
        } else {
            GNOME_BATTERY._sync = _prototypeSync;
            if (GNOME_BATTERY._indicator.icon_name == 'battery-missing-symbolic') GNOME_BATTERY._indicator.icon_name = 'system-shutdown-symbolic';
            GNOME_BATTERY._indicator.show();
        }
    }
}

const PowerIndicator = new Lang.Class({
    Name: 'PowerIndicator',

    _init: function() {
        _overrideFunction(true);
        this._settings =  Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_BATTERY, Lang.bind(this, this._updateVisibility));
        this._updateVisibility();
    },

    _updateVisibility: function() {
        if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
            GNOME_BATTERY._devicesChanged();
        } else {
            GNOME_BATTERY._sync();
        }
    },

    destroy: function() {
        _overrideFunction(false);
        this._settings.disconnect(this._connectSettings);
        this._updateVisibility();
    }
});

function getExtension() {
    return GNOME_BATTERY ? new PowerIndicator() : null;
}
