// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const Clutter = imports.gi.Clutter;
const St = imports.gi.St;

const DND = imports.ui.dnd;
const Main = imports.ui.main;
const Tweener = imports.ui.tweener;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

let _updateHotCorners;

function _overrideFunction(enabled) {
    if (enabled) {
        _updateHotCorners = imports.ui.layout.LayoutManager.prototype._updateHotCorners;
        Main.layoutManager._updateHotCorners = function() {
            (this._hotCorners || this.hotCorners).forEach(function(corner) {
                if (!corner) return;
                corner._ripple3.destroy();
                corner._ripple2.destroy();
                corner._ripple1.destroy();
                corner.destroy();
            });
            this._hotCorners ? this._hotCorners = [] : this.hotCorners = [];
        };
    } else {
        Main.layoutManager._updateHotCorners = _updateHotCorners;
    }
    Main.layoutManager._updateHotCorners();
}

const KEY_HOTCORNERS_SENSITIVITY = 'hotcorners-sensitivity';
const KEY_SHOW_HOTCORNER_LEFT = 'show-hotcorner-left';
const KEY_SHOW_HOTCORNER_RIGHT = 'show-hotcorner-right';

const HotCorner = new Lang.Class({
    Name: 'HotCorner',

    _init: function(monitor, size, style) {
        style == 'rtl' ? this.rtl = true : this.ltr = true;
        this._entered = false;
        this._monitor = monitor;
        this._x = this.rtl ? monitor.x + monitor.width : monitor.x;
        this._y = monitor.y;
        this.actor = new Clutter.Actor({ name: 'hot-corner-environs', x: this._x, y: this._y, width: size + 2, height: size + 2, reactive: true });
        this.actor.set_anchor_point_from_gravity(this.rtl ? Clutter.Gravity.NORTH_EAST : Clutter.Gravity.NORTH_WEST);
        this._corner = new Clutter.Actor({ name: 'hot-corner', width: size, height: size, opacity: 0, reactive: true });
        this._corner._delegate = this;
        this._corner.set_position(this.rtl ? this.actor.width - this._corner.width : 0, 0);
        this.actor.add_child(this._corner);
        Main.layoutManager.addChrome(this.actor);
        this._activationTime = 0;
        this.actor.connect('leave-event', Lang.bind(this, this._onEnvironsLeft));
        this.actor.connect('button-release-event', Lang.bind(this, this._onCornerClicked));
        this._corner.connect('enter-event', Lang.bind(this, this._onCornerEntered));
        this._corner.connect('button-release-event', Lang.bind(this, this._onCornerClicked));
        this._corner.connect('leave-event', Lang.bind(this, this._onCornerLeft));
        this._ripple1 = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
        this._ripple2 = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
        this._ripple3 = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
        this._ripple1.add_style_pseudo_class(style);
        this._ripple2.add_style_pseudo_class(style);
        this._ripple3.add_style_pseudo_class(style);
        let uiGroup = Main.layoutManager.uiGroup || Main.uiGroup;
        uiGroup.add_actor(this._ripple1);
        uiGroup.add_actor(this._ripple2);
        uiGroup.add_actor(this._ripple3);
    },

    _animRipple: function(ripple, delay, time, startScale, startOpacity, finalScale) {
        ripple._opacity = startOpacity;
        ripple.set_anchor_point_from_gravity(this.rtl ? Clutter.Gravity.NORTH_EAST : Clutter.Gravity.NORTH_WEST);
        ripple.visible = true;
        ripple.opacity = 255 * Math.sqrt(startOpacity);
        ripple.scale_x = ripple.scale_y = startScale;
        ripple.x = this._x;
        ripple.y = this._y;
        Tweener.addTween(ripple, { _opacity: 0, scale_x: finalScale, scale_y: finalScale, delay: delay, time: time, transition: 'linear', onUpdate: function() {
                ripple.opacity = 255 * Math.sqrt(ripple._opacity);
            }, onComplete: function() {
                ripple.visible = false;
            }
        });
    },

    _onCornerEntered: function() {
        if (!this._entered) {
            this._entered = true;
            if (!Main.overview.animationInProgress) {
                this._activationTime = Date.now() / 1000;
                this._rippleAnimation();
            }
        }
        return false;
    },

    _onCornerClicked: function() {
        if (Main.overview.animationInProgress) return true;
        if (this._activationTime == 0 || Date.now() / 1000 - this._activationTime > 0.5) Main.overview.toggle();
        return true;
    },

    _onCornerLeft: function(actor, event) {
        if (event.get_related() != this.actor) this._entered = false;
        return true;
    },

    _onEnvironsLeft: function(actor, event) {
        if (event.get_related() != this._corner) this._entered = false;
        return false;
    },

    _rippleAnimation: function() {
        this._animRipple(this._ripple1, 0.0, 0.83, 0.25, 1.0, 1.5);
        this._animRipple(this._ripple2, 0.05, 1.0, 0.0, 0.7, 1.25);
        this._animRipple(this._ripple3, 0.35, 1.0, 0.0, 0.3, 1);
        Main.overview.toggle();
    },

    destroy: function() {
        this._ripple3.destroy();
        this._ripple2.destroy();
        this._ripple1.destroy();
        this.actor.destroy();
    },

    handleDragOver: function(source, actor, x, y, time) {
        if (source != Main.xdndHandler) return DND.DragMotionResult.CONTINUE;
        if (!Main.overview.visible && !Main.overview.animationInProgress) this._rippleAnimation();
        return DND.DragMotionResult.CONTINUE;
    }
});

const HotCorners = new Lang.Class({
    Name: 'HotCorners',

    _init: function() {
        _overrideFunction(true);
        this._hotCorners = [];
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = [
            this._settings.connect('changed::'+KEY_HOTCORNERS_SENSITIVITY, Lang.bind(this, this._updateHotCorners)),
            this._settings.connect('changed::'+KEY_SHOW_HOTCORNER_LEFT, Lang.bind(this, this._onShowHotCornerLeft)),
            this._settings.connect('changed::'+KEY_SHOW_HOTCORNER_RIGHT, Lang.bind(this, this._onShowHotCornerRight))
        ];
        this._connectGlobalScreen = global.screen.connect('monitors-changed', Lang.bind(this, this._updateHotCorners));
        this._updateHotCorners();
    },

    _clearHotCorners: function() {
        this._hotCorners.forEach(function(corner) {
            if (corner) corner.destroy();
        });
        this._hotCorners = [];
    },

    _createHotCorners: function() {
        let size = this._settings.get_int(KEY_HOTCORNERS_SENSITIVITY);
        for each(let monitor in Main.layoutManager.monitors) {
            this._hotCorners.push(new HotCorner(monitor, size, 'ltr'));
            this._hotCorners.push(new HotCorner(monitor, size, 'rtl'));
        }
    },

    _onShowHotCornerLeft: function() {
        let visible = this._settings.get_boolean(KEY_SHOW_HOTCORNER_LEFT);
        for (let i = 0; i < this._hotCorners.length; i++) {
            if (this._hotCorners[i].ltr) this._hotCorners[i].actor.visible = visible;
        }
    },

    _onShowHotCornerRight: function() {
        let visible = this._settings.get_boolean(KEY_SHOW_HOTCORNER_RIGHT);
        for (let i = 0; i < this._hotCorners.length; i++) {
            if (this._hotCorners[i].rtl) this._hotCorners[i].actor.visible = visible;
        }
    },

    _updateHotCorners: function() {
        this._clearHotCorners();
        this._createHotCorners();
        this._onShowHotCornerLeft();
        this._onShowHotCornerRight();
    },

    destroy: function() {
        global.screen.disconnect(this._connectGlobalScreen);
        for each(let connection in this._connectSettings) {
            this._settings.disconnect(connection);
        }
        this._clearHotCorners();
        _overrideFunction(false);
    }
});

function getExtension() {
    return new HotCorners();
}
