// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;
const Mainloop = imports.mainloop;
const Signals = imports.signals;

const Clutter = imports.gi.Clutter;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Gtk = imports.gi.Gtk;
const Shell = imports.gi.Shell;
const St = imports.gi.St;

const Main = imports.ui.main;
const ModalDialog = imports.ui.modalDialog;
const PopupMenu = imports.ui.popupMenu;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;
const PanelTools = Me.imports.panelTools;

const Gettext = imports.gettext;
const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

const BOOKMARKS_DEFAULT = 0;
const BOOKMARKS_NONE = 0;
const BOOKMARKS_ALL = 1;
const BOOKMARKS_SUBMENU = 1;
const BOOKMARKS_MAINMENU = 2;

const DEFAULT_BOOKMARKS = [
    GLib.UserDirectory.DIRECTORY_DOCUMENTS,
    GLib.UserDirectory.DIRECTORY_DOWNLOAD,
    GLib.UserDirectory.DIRECTORY_MUSIC,
    GLib.UserDirectory.DIRECTORY_PICTURES,
    GLib.UserDirectory.DIRECTORY_VIDEOS
];

const DEFAULT_DIRECTORIES = [
    GLib.UserDirectory.DIRECTORY_DESKTOP,
    GLib.UserDirectory.DIRECTORY_DOCUMENTS,
    GLib.UserDirectory.DIRECTORY_DOWNLOAD,
    GLib.UserDirectory.DIRECTORY_MUSIC,
    GLib.UserDirectory.DIRECTORY_PICTURES,
    GLib.UserDirectory.DIRECTORY_PUBLIC_SHARE,
    GLib.UserDirectory.DIRECTORY_TEMPLATES,
    GLib.UserDirectory.DIRECTORY_VIDEOS
];

const KEY_BOOKMARKS_SCOPE = 'bookmarks-scope';
const KEY_SHOW_BOOKMARKS = 'show-bookmarks';
const KEY_SHOW_BOOKMARKS_CUSTOM = 'show-bookmarks-custom';
const KEY_SHOW_GNOME_GLOBE = 'show-gnome-globe';
const KEY_SHOW_GNOME_SEARCH_TOOL = 'show-gnome-search-tool';
const KEY_SHOW_NETWORK_PLACES = 'show-network-places';
const KEY_SHOW_PLACES = 'show-places';
const KEY_SHOW_RECENT_DOCUMENTS = 'show-recent-documents';
const KEY_SHOW_REMOVABLE_DEVICES = 'show-removable-devices';

const ModalPlaceDialog = new Lang.Class({
    Name: 'ModalPlaceDialog',
    Extends: ModalDialog.ModalDialog,

    _init: function(subject, description, buttonLabel, callback) {
        this.parent({ styleClass: 'show-processes-dialog' });
        let mainContentBox = new St.BoxLayout();
        this.contentLayout.add(mainContentBox, { x_fill: true, y_fill: true });
        this.icon = new St.Icon({ icon_name: 'dialog-warning' });
        if (St.IconType) this.icon.icon_type = St.IconType.FULLCOLOR;
        mainContentBox.add(this.icon, { x_fill: true, y_fill: false, x_align: St.Align.END, y_align: St.Align.START });
        let messageBox = new St.BoxLayout({ vertical: true });
        mainContentBox.add(messageBox, { y_align: St.Align.START });
        messageBox.add(new St.Label({ text: subject, style_class: 'show-processes-dialog-subject' }), { y_fill: false, y_align: St.Align.START });
        messageBox.add(new St.Label({ text: description, style_class: 'show-processes-dialog-description' }), { y_fill: true, y_align: St.Align.START });
        this.setButtons([
            { label: _("Cancel"), action: Lang.bind(this, function() { this.close(); }), key: Clutter.Escape },
            { label: buttonLabel, action: Lang.bind(this, function() { this.close(); callback(); })}
        ]);
    }
});

const PlaceInfo = new Lang.Class({
    Name: 'PlaceInfo',
    
    _init: function(category, file, label, icon) {
        this._category = category;
        this._file = file;
        this._name = label || this.getLabel();
        this._icon = icon ? new Gio.ThemedIcon({ name: icon }) : this.getIcon();
    },

    getIcon: function() {
        try {
            let info = this._file.query_info('standard::icon', 0, null);
            return info.get_icon();
        } catch(e) {
            switch (this._category) {
            case 'network':
                return new Gio.ThemedIcon({ name: 'folder-remote' });
            case 'devices':
                return new Gio.ThemedIcon({ name: 'drive-harddisk' });
            case 'special':
            case 'bookmarks':
            default:
                if (!this._file.is_native()) return new Gio.ThemedIcon({ name: 'folder-remote' });
                return new Gio.ThemedIcon({ name: 'folder' });
            }
        }
    },

    getLabel: function() {
        try {
            return this._file.query_info('standard::display-name', 0, null).get_display_name();
        } catch(e if e instanceof Gio.IOErrorEnum) {
            return this._file.get_basename();
        }
    },

    launch: function() {
        let event = Clutter.get_current_event();
        let launchContext = global.create_app_launch_context(event.get_time(), -1);
        try {
            Gio.app_info_launch_default_for_uri(this._file.get_uri(), launchContext);
        } catch(e if e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.NOT_MOUNTED)) {
            this._file.mount_enclosing_volume(0, null, null, function(file, result) {
                file.mount_enclosing_volume_finish(result);
                Gio.app_info_launch_default_for_uri(file.get_uri(), launchContext);
            });
        } catch(e) {
            Main.notifyError(_("Failed to launch '%s'").format(this._name), e.message);
        }
    }
});

const PlaceDeviceInfo = new Lang.Class({
    Name: 'PlaceDeviceInfo',
    Extends: PlaceInfo,

    _init: function(category, mount) {
        this._mount = mount;
        this.parent(category, mount.get_root(), mount.get_name());
    },

    _removeFinish: function(o, res, data) {
        try {
            if (this._mount.can_eject()) {
                this._mount.eject_finish(res);
            } else {
                this._mount.unmount_finish(res);
            }
        } catch (e) {
            let message = _("Failed to unmount '%s'").format(o.get_name());
            Main.overview.setMessage(message, Lang.bind(this, this.remove), _("Retry"));
        }
    },

    getIcon: function() {
        return this._mount.get_icon();
    },

    isRemovable: function() {
        return this._mount.can_eject() ||this._mount.can_unmount();
    },

    remove: function() {
        if (!this.isRemovable()) return;
        if (this._mount.can_eject()) {
            this._mount.eject(0, null, Lang.bind(this, this._removeFinish));
        } else if (this._mount.can_unmount()) {
            this._mount.unmount(0, null, Lang.bind(this, this._removeFinish));
        }
    }
});

const PlacesManager = new Lang.Class({
    Name: 'PlacesManager',

    _init: function() {
        this._places = {
            special: [],
            directories: [],
            bookmarks: [],
            custom: [],
            volumes: [],
            devices: [],
            network: []
        };
        this._bookmarksPath = GLib.build_filenamev([GLib.get_home_dir(), '.gtk-bookmarks']);
        if (!GLib.file_test(this._bookmarksPath, GLib.FileTest.EXISTS)) {
            this._bookmarksPath = GLib.build_filenamev([GLib.get_home_dir(), '.config/gtk-3.0/bookmarks']);
        }
        this._bookmarkTimeoutId = 0;
        let bookmarksFile = Gio.file_new_for_path(this._bookmarksPath);
        this._bookmarksMonitor = bookmarksFile.monitor_file(Gio.FileMonitorFlags.NONE, null);
        this._connectBookmarksMonitor = this._bookmarksMonitor.connect('changed', Lang.bind(this, function () {
            if (this._bookmarkTimeoutId > 0) return;
            this._bookmarkTimeoutId = Mainloop.timeout_add(100, Lang.bind(this, function () {
                this._bookmarkTimeoutId = 0;
                this._update();
                return false;
            }));
        }));
        this._volumeMonitor = Gio.VolumeMonitor.get();
        this._connectVolumeMonitor = [
            this._volumeMonitor.connect('volume-added', Lang.bind(this, this._update)),
            this._volumeMonitor.connect('volume-removed',Lang.bind(this, this._update)),
            this._volumeMonitor.connect('volume-changed', Lang.bind(this, this._update)),
            this._volumeMonitor.connect('mount-added', Lang.bind(this, this._update)),
            this._volumeMonitor.connect('mount-removed', Lang.bind(this, this._update)),
            this._volumeMonitor.connect('mount-changed', Lang.bind(this, this._update)),
            this._volumeMonitor.connect('drive-connected', Lang.bind(this, this._update)),
            this._volumeMonitor.connect('drive-disconnected', Lang.bind(this, this._update)),
            this._volumeMonitor.connect('drive-changed', Lang.bind(this, this._update))
        ];
        this._update();
    },

    _addMount: function(category, mount) {
        this._places[category].push(new PlaceDeviceInfo(category, mount));
    },

    _update: function() {
        this._places.special = [];
        this._places.custom = [];
        this._places.bookmarks = [];
        this._places.directories = [];
        this._places.volumes = [];
        this._places.devices = [];
        this._places.network = [];
        this._places.special.push(new PlaceInfo('special', Gio.File.new_for_path(GLib.get_home_dir()), _("Home Folder")));
        this._places.volumes.push(new PlaceInfo('special', Gio.File.new_for_uri('computer:///'), _("Computer")));
        this._places.volumes.push(new PlaceInfo('special', Gio.File.new_for_uri('file:///'), _("File System")));
        this._places.volumes.push(new PlaceInfo('special', Gio.File.new_for_uri('network:///'), _("Network")));
        for (let i = 0; i < DEFAULT_BOOKMARKS.length; i++) {
            let specialPath = GLib.get_user_special_dir(DEFAULT_BOOKMARKS[i]);
            if (!GLib.file_test(specialPath, GLib.FileTest.EXISTS)) continue;
            if (specialPath == GLib.get_home_dir()) continue;
            let file = Gio.File.new_for_path(specialPath);
            this._places.bookmarks.push(new PlaceInfo('special', file));
        }
        for (let i = 0; i < DEFAULT_DIRECTORIES.length; i++) {
            let specialPath = GLib.get_user_special_dir(DEFAULT_DIRECTORIES[i]);
            if (!GLib.file_test(specialPath, GLib.FileTest.EXISTS)) continue;
            if (specialPath == GLib.get_home_dir()) continue;
            let file = Gio.File.new_for_path(specialPath);
            this._places.directories.push(new PlaceInfo('special', file));
        }
        if (GLib.file_test(this._bookmarksPath, GLib.FileTest.EXISTS)) {
            let bookmarksContent = Shell.get_file_contents_utf8_sync(this._bookmarksPath);
            let bookmarks = bookmarksContent.split('\n');
            for (let i = 0; i < bookmarks.length; i++) {
                let bookmarkLine = bookmarks[i];
                let components = bookmarkLine.split(' ');
                let bookmark = components[0];
                if (!bookmark) continue;
                let file = Gio.File.new_for_uri(bookmark);
                let duplicate = false;
                for (let i = 0; i < this._places.special.length; i++) {
                    if (file.equal(this._places.special[i]._file)) {
                        duplicate = true;
                        break;
                    }
                }
                if (duplicate) continue;
                for (let i = 0; i < this._places.bookmarks.length; i++) {
                    if (file.equal(this._places.bookmarks[i]._file)) {
                        duplicate = true;
                        break;
                    }
                }
                if (duplicate) continue;
                for (let i = 0; i < this._places.directories.length; i++) {
                    if (file.equal(this._places.directories[i]._file)) {
                        duplicate = true;
                        break;
                    }
                }
                if (duplicate) continue;
                this._places.custom.push(new PlaceInfo('bookmarks', file, components[1] || ''));
            }
        }
        let drives = this._volumeMonitor.get_connected_drives();
        for (let i = 0; i < drives.length; i++) {
            let volumes = drives[i].get_volumes();
            for (let j = 0; j < volumes.length; j++) {
                let category = 'devices';
                let mount = volumes[j].get_mount();
                if (mount != null) this._addMount(category, mount);
            }
        }
        let volumes = this._volumeMonitor.get_volumes();
        for (let i = 0; i < volumes.length; i++) {
            if (volumes[i].get_drive() != null || volumes[i].get_identifier('class') == null) continue;
            let category = 'devices';
            if (volumes[i].get_identifier('class').indexOf('network') >= 0) category = 'network';
            let mount = volumes[i].get_mount();
            if (mount != null) this._addMount(category, mount);
        }
        let mounts = this._volumeMonitor.get_mounts();
        for (let i = 0; i < mounts.length; i++) {
            if (mounts[i].is_shadowed()) continue;
            if (mounts[i].get_volume()) continue;
            let root = mounts[i].get_default_location();
            let category = 'network';
            //if (root.is_native()) category = 'devices';
            this._addMount(category, mounts[i]);
        }
        this.emit('places-updated');
    },

    getPlaces: function (category) {
        this._places[category].sort(function(a, b) {
            return Convenience.getSorted(a._name, b._name);
        });
        return this._places[category];
    },

    destroy: function() {
        for each(let connection in this._connectVolumeMonitor) {
            this._volumeMonitor.disconnect(connection);
        }        
        this._bookmarksMonitor.disconnect(this._connectBookmarksMonitor);
    }
});
Signals.addSignalMethods(PlacesManager.prototype);

const PopupPlaceMenuItem = new Lang.Class({
    Name: 'PopupPlaceMenuItem',
    Extends: PanelTools.PopupMenuItem,

    _init: function(place, placesMenu) {
        this.parent(place._name, true);
        this.icon.gicon = place._icon;
        if (place.isRemovable) {
            this.addButton('media-eject-symbolic', Lang.bind(this, function() {
                if (placesMenu.isOpen) placesMenu.close(0);
                Main.overview.hide();
                new ModalPlaceDialog(_("Eject the selected volume?"),
                    _("Volume: '%s'").format(place._name.toUpperCase()),
                    _("Eject"),
                    Lang.bind(this, function() {
                        place.remove();
                    })
                ).open();
            }));
        }
        this.connect('activate', Lang.bind(this, function() {
            Main.overview.hide();
            place.launch();
        }));
    }
});

const PopupRecentMenuItem = new Lang.Class({
    Name: 'PopupRecentMenuItem',
    Extends: PanelTools.PopupMenuItem,

    _init: function(file) {
        this.parent(PanelTools.getStringCutOut(file.get_display_name()), true);
        this.icon.gicon = Gio.content_type_get_icon(file.get_mime_type());
        this.connect('activate', Lang.bind(this, function() {
            let event = Clutter.get_current_event();
            let launchContext = global.create_app_launch_context(event.get_time(), -1);
            Main.overview.hide();
            Gio.app_info_launch_default_for_uri(file.get_uri(), launchContext);
        }));
    }
});

const PlacesButton = new Lang.Class({
    Name: 'PlacesButton',
    Extends: PanelTools.Button,

    _init: function() {
        this.parent(1.0, _("Places"), { icon_name: 'folder-symbolic', text: _("Places") });
        this._itemBookmarks = new PopupMenu.PopupMenuSection();
        this.menu.addMenuItem(this._itemBookmarks);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemDevices = new PopupMenu.PopupMenuSection();
        this.menu.addMenuItem(this._itemDevices);
        this._itemDevicesVolumes = new PopupMenu.PopupMenuSection();
        this._itemDevices.addMenuItem(this._itemDevicesVolumes);
        this._itemDevicesRemovableDevices = new PanelTools.PopupSubMenuMenuItem(_("Removable Devices"), true);
        this._itemDevicesRemovableDevices.setIconName('media-removable');
        this._itemDevices.addMenuItem(this._itemDevicesRemovableDevices);
        this._itemDevicesNetworkPlaces = new PanelTools.PopupSubMenuMenuItem(_("Network Places"), true);
        this._itemDevicesNetworkPlaces.setIconName('network');
        this._itemDevices.addMenuItem(this._itemDevicesNetworkPlaces);
        this._separatorApplications = new PopupMenu.PopupSeparatorMenuItem();
        this.menu.addMenuItem(this._separatorApplications);
        this._itemApplications = new PopupMenu.PopupMenuSection();
        this.menu.addMenuItem(this._itemApplications);
        this._separatorRecent = new PopupMenu.PopupSeparatorMenuItem();
        this.menu.addMenuItem(this._separatorRecent);
        this._itemRecent = new PopupMenu.PopupMenuSection();
        this.menu.addMenuItem(this._itemRecent);
        let item = new PanelTools.PopupMenuItem(_("Clear Recent Documents..."), true);
        item.setIconName('edit-clear');
        item.connect('activate', Lang.bind(this, function() {
            Main.overview.hide();
            new ModalPlaceDialog(_("Clear the Recent Documents list?"),
                _("If you clear the Recent Documents list, you clear the following:\n\u2022 All items from the Places \u2192 Recent Documents menu item.\n\u2022 All items from the recent documents list in all applications."),
                _("Clear list"),
                Lang.bind(this, function() {
                    this._recentManager.purge_items();
                    this._itemRecent.actor.hide();
                    this._separatorRecent.actor.hide();
                })
            ).open();
        }));
        this._itemRecent.addMenuItem(item);
        this._itemRecentDocuments = new PanelTools.PopupSubMenuMenuItem(_("Recent Documents"), true);
        this._itemRecentDocuments.setIconName('document-open-recent');
        this._itemRecent.addMenuItem(this._itemRecentDocuments);
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = [
            this._settings.connect('changed::'+KEY_BOOKMARKS_SCOPE, Lang.bind(this, this._updateBookmarks)),
            this._settings.connect('changed::'+KEY_SHOW_BOOKMARKS, Lang.bind(this, this._updateBookmarks)),
            this._settings.connect('changed::'+KEY_SHOW_BOOKMARKS_CUSTOM, Lang.bind(this, this._updateBookmarks)),
            this._settings.connect('changed::'+KEY_SHOW_GNOME_GLOBE, Lang.bind(this, this._onShowApplications)),
            this._settings.connect('changed::'+KEY_SHOW_GNOME_SEARCH_TOOL, Lang.bind(this, this._onShowApplications)),
            this._settings.connect('changed::'+KEY_SHOW_NETWORK_PLACES, Lang.bind(this, this._onShowDevices)),
            this._settings.connect('changed::'+KEY_SHOW_PLACES, Lang.bind(this, this._onShowPlaces)),
            this._settings.connect('changed::'+KEY_SHOW_RECENT_DOCUMENTS, Lang.bind(this, this._onShowRecentDokuments)),
            this._settings.connect('changed::'+KEY_SHOW_REMOVABLE_DEVICES, Lang.bind(this, this._onShowDevices))
        ];
        this._onShowPlaces();
        this._appSystem = Shell.AppSystem.get_default();
        this._connectAppSystem = this._appSystem.connect('installed-changed', Lang.bind(this, this._updateApplications));
        this._updateApplications();
        this._placesManager = new PlacesManager();
        this._connectPlacesManager = this._placesManager.connect('places-updated', Lang.bind(this, this._updatePlaces));
        this._updatePlaces();
        this._recentManager = new Gtk.RecentManager();
        this.menu.connect('open-state-changed', Lang.bind(this, this._updateRecentDokuments));
        this._updateRecentDokuments();
        this.actor.connect('destroy', Lang.bind(this, function() {
            this._placesManager.disconnect(this._connectPlacesManager);
            this._placesManager.destroy();
            this._appSystem.disconnect(this._connectAppSystem);
            for each(let connection in this._connectSettings) {
                this._settings.disconnect(connecttion);
            }
        }));
    },

    _onShowApplications: function() {
        if (this._itemGnomeGlobe) this._itemGnomeGlobe.actor.visible = this._settings.get_boolean(KEY_SHOW_GNOME_GLOBE);
        if (this._itemGnomeSearchTool) this._itemGnomeSearchTool.actor.visible = this._settings.get_boolean(KEY_SHOW_GNOME_SEARCH_TOOL);
        let visible = (this._itemGnomeGlobe && this._itemGnomeGlobe.actor.visible) || (this._itemGnomeSearchTool && this._itemGnomeSearchTool.actor.visible);
        this._itemApplications.actor.visible = visible;
        this._separatorApplications.actor.visible = visible;
    },

    _onShowDevices: function() {
        this._itemDevicesVolumes.actor.visible = this._itemDevicesVolumes.numMenuItems > 0;
        this._itemDevicesRemovableDevices.actor.visible = this._settings.get_boolean(KEY_SHOW_NETWORK_PLACES) && this._itemDevicesRemovableDevices.menu.numMenuItems > 0;
        this._itemDevicesNetworkPlaces.actor.visible = this._settings.get_boolean(KEY_SHOW_REMOVABLE_DEVICES) && this._itemDevicesNetworkPlaces.menu.numMenuItems > 0;
        this._itemDevices.actor.visible = this._itemDevicesVolumes.actor.visible || this._itemDevicesRemovableDevices.actor.visible || this._itemDevicesNetworkPlaces.actor.visible;
    },

    _onShowPlaces: function() {
        this.updateButton(this._settings.get_enum(KEY_SHOW_PLACES));
        PanelTools.updatePanelCorner(this, 'left');
    },

    _onShowRecentDokuments: function() {
        let visible = this._settings.get_boolean(KEY_SHOW_RECENT_DOCUMENTS) && this._itemRecentDocuments.menu.numMenuItems > 0;
        this._itemRecent.actor.visible = visible;
        this._separatorRecent.actor.visible = visible;
    },

    _updateApplications: function() {
        this._itemApplications.removeAll();
        let app = PanelTools.checkApplication('nautilus-connect-server.desktop');
        if (app) {
            this._itemGnomeGlobe = new PanelTools.PopupApplicationMenuItem(app, true);
            this._itemApplications.addMenuItem(this._itemGnomeGlobe);
        } else {
            if (GLib.find_program_in_path('nautilus-connect-server')) {
                this._itemGnomeGlobe = new PanelTools.PopupMenuItem(_("Connect to Server..."), true);
                this._itemGnomeGlobe.setIconName('gnome-globe');
                this._itemGnomeGlobe.connect('activate', Lang.bind(this, function() {
                    Main.overview.hide();
                    Main.Util.spawnCommandLine('nautilus-connect-server');
                }));
                this._itemApplications.addMenuItem(this._itemGnomeGlobe);
            }
        }
        let app = PanelTools.checkApplication('gnome-search-tool.desktop');
        if (app) {
            this._itemGnomeSearchTool = new PanelTools.PopupApplicationMenuItem(app, true);
            this._itemApplications.addMenuItem(this._itemGnomeSearchTool);
        }
        this._onShowApplications();
    },

    _updateBookmarks: function() {
        this._itemBookmarks.removeAll();
        let bookmarksScope = this._placesManager.getPlaces(this._settings.get_enum(KEY_BOOKMARKS_SCOPE) == BOOKMARKS_DEFAULT ? 'bookmarks' : 'directories');
        let keyShowBookmarks = this._settings.get_enum(KEY_SHOW_BOOKMARKS);
        let keyShowBookmarksCustom = this._settings.get_enum(KEY_SHOW_BOOKMARKS_CUSTOM);
        let bookmarksCustom = this._placesManager.getPlaces('custom');
        if (keyShowBookmarksCustom == BOOKMARKS_MAINMENU && bookmarksCustom.length > 0) {
            for (let i = 0; i < bookmarksCustom.length; i++) {
                this._itemBookmarks.addMenuItem(new PopupPlaceMenuItem(bookmarksCustom[i]));
            }
            this._itemBookmarks.addMenuItem(new PopupMenu.PopupSeparatorMenuItem);
        }
        let specialSection = this._placesManager.getPlaces('special');
        for (let i = 0; i < specialSection.length; i++) {
            this._itemBookmarks.addMenuItem(new PopupPlaceMenuItem(specialSection[i]));
        }
        if (keyShowBookmarks == BOOKMARKS_MAINMENU && bookmarksScope.length > 0) {
            for (let i = 0; i < bookmarksScope.length; i++) {
                this._itemBookmarks.addMenuItem(new PopupPlaceMenuItem(bookmarksScope[i]));
            }
        }
        let item = new PanelTools.PopupSubMenuMenuItem(_("Bookmarks"), true);
        item.setIconName('user-bookmarks');
        if (keyShowBookmarks == BOOKMARKS_SUBMENU && bookmarksScope.length > 0) {
            for (let i = 0; i < bookmarksScope.length; i++) {
                item.menu.addMenuItem(new PopupPlaceMenuItem(bookmarksScope[i]));
            }
        }
        if (keyShowBookmarksCustom == BOOKMARKS_SUBMENU && bookmarksCustom.length > 0) {
            for (let i = 0; i < bookmarksCustom.length; i++) {
                item.menu.addMenuItem(new PopupPlaceMenuItem(bookmarksCustom[i]));
            }
        }
        if (item.menu.length > 0) this._itemBookmarks.addMenuItem(item);
    },

    _updateDevices: function() {
        this._itemDevicesVolumes.removeAll();
        let volumes = this._placesManager.getPlaces('volumes');
        for (let i = 0; i < volumes.length; i++) {
            this._itemDevicesVolumes.addMenuItem(new PopupPlaceMenuItem(volumes[i]));
        }
        this._itemDevicesRemovableDevices.menu.removeAll();
        let devices = this._placesManager.getPlaces('devices');
        if (devices.length > 0) {
            for (let i = 0; i < devices.length; i++) {
                this._itemDevicesRemovableDevices.menu.addMenuItem(new PopupPlaceMenuItem(devices[i], this.menu));
            }
        }
        this._itemDevicesNetworkPlaces.menu.removeAll();
        let network = this._placesManager.getPlaces('network');
        if (network.length > 0) {
            for (let i = 0; i < network.length; i++) {
                this._itemDevicesNetworkPlaces.menu.addMenuItem(new PopupPlaceMenuItem(network[i], this.menu));
            }
        }
        this._onShowDevices();
    },

    _updatePlaces: function() {
        this._updateBookmarks();
        this._updateDevices();
    },

    _updateRecentDokuments: function() {
        this._itemRecentDocuments.menu.removeAll();
        if (this._recentManager.size > 0) {
            let items = this._recentManager.get_items();
            items.sort(function(a, b) {
                return Convenience.getSorted(a.get_display_name(), b.get_display_name());
            });
            for (let i = 0; i < items.length; i++) {
                if (GLib.file_test(items[i].get_uri_display(), GLib.FileTest.IS_REGULAR)) this._itemRecentDocuments.menu.addMenuItem(new PopupRecentMenuItem(items[i]));
            }
        }
        this._onShowRecentDokuments();
    }
});

function getExtension() {
    return PanelTools.addToStatusArea('dschaerfPlaces', new PlacesButton(), 1, 'left');
}
