// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;
const Mainloop = imports.mainloop;

const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Meta = imports.gi.Meta;
const Shell = imports.gi.Shell;
const St = imports.gi.St;

const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;
const UserMenu = imports.ui.userMenu;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;
const PanelTools = Me.imports.panelTools;

const Gettext = imports.gettext;
const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

function _canHibernate(asyncCallback) {
    if (this._proxy) {
        this._proxy.call("CanHibernate", null, Gio.DBusCallFlags.NONE, -1, null, function(proxy, asyncResult) {
            try {
                let result = proxy.call_finish(asyncResult).deep_unpack();
                asyncCallback(result[0] != 'no');
            } catch(e) {
                asyncCallback(false);
            }
        });
    } else {
        Mainloop.idle_add(Lang.bind(this, function() {
            asyncCallback(false);
            return false;
        }));
    }
}

function _getUserMenu() {
    if (Convenience.checkGSV(3, 4)) {
        try { return Main.panel._statusArea.userMenu; } catch(e) { return null; }
    } else {
        try { return Main.panel.statusArea.userMenu; } catch(e) { return null; }
    }
}

function _onGNOMEShellRestartActivate() {
    global.reexec_self();
}

function _onHibernateActivate() {
    this.menu.close(0);
    Main.overview.hide();
    if (Convenience.checkGSV(3, 4)) {
        this._screenSaverProxy.LockRemote(Lang.bind(this, function() {
            this._upClient.hibernate_sync(null);
        }));
    } else if (Convenience.checkGSV(3, 6)) {
        if (this._screenSaverSettings.get_boolean('lock-enabled')) {
            let tmpId = Main.screenShield.connect('lock-screen-shown', Lang.bind(this, function() {
                Main.screenShield.disconnect(tmpId);
                this._upClient.hibernate_sync(null);
            }));
            Main.screenShield.lock(true);
        } else {
            this._upClient.hibernate_sync(null);
        }
    } else {
        if (this._loginManager._proxy) {
            this._loginManager._proxy.call("Hibernate", GLib.Variant.new('(b)', [true]), Gio.DBusCallFlags.NONE, -1, null, null);
        } else {
            this._loginManager.emit('prepare-for-sleep', true);
            this._loginManager.emit('prepare-for-sleep', false);
        }
    }
}

function _onPanelManagerSettingsActivate() {
    this.menu.close(0);
    Main.overview.hide();
    Main.Util.spawnCommandLine('gnome-shell-extension-prefs '+Me.metadata['uuid']);
}

function _onPowerOffActivate() {
    this.menu.close(0);
    Main.overview.hide();
    if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6)) {
        this._session.ShutdownRemote();
    } else {
        this._loginManager.listSessions(Lang.bind(this, function(result) {
            let sessions = [];
            let n = 0;
            for (let i = 0; i < result.length; i++) {
                let[id, uid, userName, seat, sessionPath] = result[i];
                let proxy = new UserMenu.SystemdLoginSession(Gio.DBus.system, 'org.freedesktop.login1', sessionPath);
                if (proxy.Class != 'user') continue;
                if (proxy.State == 'closing') continue;
                if (proxy.Id == GLib.getenv('XDG_SESSION_ID')) continue;
                sessions.push({ user: this._userManager.get_user(userName), username: userName, info: { type: proxy.Type, remote: proxy.Remote }});
                n++;
                if (n == 5) break;
            }
            if (n != 0) {
                this._openSessionWarnDialog(sessions);
            } else {
                this._session.ShutdownRemote();
            }
        }));
    }
}

function _onSuspendActivate() {
    this.menu.close(0);
    Main.overview.hide();
    if (Convenience.checkGSV(3, 4)) {
        this._screenSaverProxy.LockRemote(Lang.bind(this, function() {
            this._upClient.suspend_sync(null);
        }));
    } else if (Convenience.checkGSV(3, 6)) {
        if (this._screenSaverSettings.get_boolean('lock-enabled')) {
            let tmpId = Main.screenShield.connect('lock-screen-shown', Lang.bind(this, function() {
                Main.screenShield.disconnect(tmpId);
                this._upClient.suspend_sync(null);
            }));
            Main.screenShield.lock(true);
        } else {
            this._upClient.suspend_sync(null);
        }
    } else {
        this._loginManager.suspend();
    }
}

function _onTerminalActivate() {
    this.menu.close(0);
    Main.overview.hide();
    Main.Util.spawnCommandLine('gnome-terminal');
}

function _updateHibernate(actor) {
    if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6)) {
        actor.visible = this._upClient.get_can_hibernate();
    } else {
        _canHibernate.call(this._loginManager, Lang.bind(this, function(result) {
            actor.visible = result;
        }));
    }
}

function _updateLogout(actor) {
    this._updateLogout();
    actor.visible = this._logoutItem.actor.visible;
}

function _updateLockScreen(actor) {
    this._updateLockScreen();
    actor.visible = this._lockScreenItem.actor.visible;
}

function _updatePowerOff(actor) {
    this._updateSuspendOrPowerOff();
    actor.visible = this._suspendOrPowerOffItem.label.indexOf(Gettext.gettext("Power Off")) >= 0;
}

function _updateSuspend(actor) {
    this._updateSuspendOrPowerOff();
    actor.visible = this._suspendOrPowerOffItem.label.indexOf(Gettext.gettext("Suspend")) >= 0;
}

function _updateSwitchUser(item) {
    this._updateSwitchUser();
    item.actor.visible = this._loginiScreenItem.actor.visible;
}

const DEFAULT_BASIC_SETTINGS = [
    [ _("System Settings"), 'gnome-control-center', 'preferences-system' ],
    [ _("GNOME Shell Extensions"), 'gnome-shell-extension-prefs', 'preferences-other' ],
    [ _("Startup Applications"), 'gnome-session-properties', 'session-properties' ],
    [ _("GNOME Tweak Tool"), 'gnome-tweak-tool', 'preferences-system' ],
    [ _("Synaptic Package Manager"), 'synaptic-pkexec', 'synaptic' ],
    [ _("Configure printers"), 'system-config-printer', 'printer' ],
    [ _("Users Settings"), 'users-admin', 'config-users' ]
];

const DEFAULT_COMMUNICATION_APPLICATIONS = [
    'amsn.desktop',
    'emesene.desktop',
    'empathy.desktop',
    'gajim.desktop',
    'gwibber.desktop',
    'licq.desktop',
    'pidgin.desktop',
    'psi.desktop',
    'qutim.desktop',
    'skype.desktop',
    'xchat.desktop'
];

const DEFAULT_EXTENSION = [
    [ _("Settings for PanelManager"), Me.metadata['uuid'], 'preferences-other' ],
    [ _("Settings for Temperature"), 'temperature@extensions.dschaerf.de', 'preferences-other' ],
    [ _("Settings for Weather"), 'weather@extensions.dschaerf.de', 'preferences-other' ]
];

const GNOME_USERMENU = _getUserMenu();

const KEY_SHOW_STATUSAREA = 'show-statusarea';
const KEY_SHOW_STATUSMENU = 'show-statusmenu';

const STATUSAREA_DEFAULT = 0;
const STATUSAREA_ADVANCED = 1;
const STATUSAREA_ALTERNATE = 2;

const STATUSMENU_DEFAULT = 0;
const STATUSMENU_ICON = 1;

const PopupBasicSettingsMenuSection = new Lang.Class({
    Name: 'PopupBasicSettingsMenuSection',
    Extends: PopupMenu.PopupMenuSection,

    _init: function(wantIcon) {
        this.parent();
        this._wantIcon = wantIcon || false;
        this._appSystem = Shell.AppSystem.get_default();
        this._connectAppSystem = this._appSystem.connect('installed-changed', Lang.bind(this, this._onInstalledChanged));
        this._onInstalledChanged();
        this.actor.connect('destroy', Lang.bind(this, function() {
            this._appSystem.disconnect(this._connectAppSystem);
        }));
    },

    _onInstalledChanged: function() {
        this.removeAll();
        let apps = [];
        for each(let app in DEFAULT_BASIC_SETTINGS) {
            if (GLib.find_program_in_path(app[1])) apps.push(app);
        }
        apps.sort(function(a, b) {
            return Convenience.getSorted(a[0], b[0]);
        });
        for (let i = 0; i < apps.length; i++) {
            let app = apps[i];
            let item = new PanelTools.PopupMenuItem(app[0], this._wantIcon);
            if (this._wantIcon) item.setIconName(app[2]);
            item.connect('activate', Lang.bind(this, function() {
                Main.overview.hide();
                Main.Util.spawnCommandLine(app[1]);
            }));
            this.addMenuItem(item);
        }
    }
});

const PopupCommunicationApplicationsMenuSection = new Lang.Class({
    Name: 'PopupCommunicationApplicationsMenuSection',
    Extends: PopupMenu.PopupMenuSection,

    _init: function(wantIcon, wantOnlineAccounts) {
        this.parent();
        this._wantIcon = wantIcon || false;
        this._wantOnlineAccounts = wantOnlineAccounts || false;
        this._appSystem = Shell.AppSystem.get_default();
        this._connectAppSystem = this._appSystem.connect('installed-changed', Lang.bind(this, this._onInstalledChanged));
        this._onInstalledChanged();
        this.actor.connect('destroy', Lang.bind(this, function() {
            this._appSystem.disconnect(this._connectAppSystem);
        }));
    },

    _onInstalledChanged: function() {
        this.removeAll();
        let apps = [];
        for each(let app in DEFAULT_COMMUNICATION_APPLICATIONS) {
            app = PanelTools.checkApplication(app);
            if (app) apps.push(app);
        }
        apps.sort(function(a, b) {
            return a.compare_by_name(b);
        });
        for each(let app in apps) {
            this.addMenuItem(new PanelTools.PopupApplicationMenuItem(app, this._wantIcon));
        }
        this.actor.visible = this.numMenuItems > 0;
        if (this._wantOnlineAccounts) {
            let app = PanelTools.checkApplication('gnome-online-accounts-panel.desktop');
            if (app) {
                this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
                this.addMenuItem(new PanelTools.PopupApplicationMenuItem(app));
            }
        }
    }
});

const PopupIMStatusChooserMenuItem = new Lang.Class({
    Name: 'PopupIMStatusChooserMenuItem',
    Extends: UserMenu.IMStatusChooserItem,

    _init: function(icon) {
        this.icon = icon || null;
        this.parent();
        this.removeActor(this._iconBin);
        this.removeActor(this._section.actor);
        this._hbox = new St.BoxLayout({ style_class: 'paneltools-popup-menu-box' });
        this.addActor(this._hbox, { expand: true });
        this._hbox.add(this._iconBin);
        this._hbox.add(new St.Bin(), { expand: true });
        this._hbox.add(this._section.actor, { align: St.Align.END });
        this.connect('activate', Lang.bind(GNOME_USERMENU, GNOME_USERMENU._onMyAccountActivate));
        this._connectAccountMrg = GNOME_USERMENU._accountMgr.connect('most-available-presence-changed', Lang.bind(this, this._updatePresenceStatus));
        GNOME_USERMENU._accountMgr.prepare_async(null, Lang.bind(this, this._updatePresenceStatus))
        this.actor.connect('destroy', Lang.bind(this, function() {
            GNOME_USERMENU._accountMgr.disconnect(this._connectAccountMrg);
        }));
    },

    _updatePresenceStatus: function() {
        if (this.icon) this.icon.icon_name = GNOME_USERMENU._iconBox.get_child().get_icon_name();
    }
});

const PopupNotificationsSwitchMenuItem = new Lang.Class({
    Name: 'PopupNotificationsSwitchMenuItem',
    Extends: PopupMenu.PopupSwitchMenuItem,

    _init: function() {
        this.parent(_("Notifications"));
        this.removeActor(this.label);
        this.removeActor(this._statusBin);
        this._hbox = new St.BoxLayout({ style_class: 'paneltools-popup-menu-box' });
        this.addActor(this._hbox, { expand: true });
        this._hbox.add(this.label);
        this._hbox.add(new St.Bin(), { expand: true });
        this._hbox.add(this._statusBin, { align: St.Align.END });
        this.connect('toggled', Lang.bind(this, function() {
            GNOME_USERMENU._notificationsSwitch.toggle();
        }));
        this._updateNotificationsSwitch();
        this._connectPresence = GNOME_USERMENU._presence.connectSignal('StatusChanged', Lang.bind(this, this._updateNotificationsSwitch));
        this.actor.connect('destroy', Lang.bind(this, function() {
            GNOME_USERMENU._presence.disconnectSignal(this._connectPresence);
        }));
    },

    _updateNotificationsSwitch: function() {
        this.setToggleState(GNOME_USERMENU._notificationsSwitch._switch.state);
    }
});

const PopupUserMenuAdvancedMenuSection = new Lang.Class({
    Name: 'PopupUserMenuAdvancedMenuSection',
    Extends: PopupMenu.PopupMenuSection,

    _init: function() {
        this.parent();
        this._itemLockScreen = new PanelTools.PopupMenuItem(_("Lock Screen"));
        this._itemLockScreen.connect('activate', Lang.bind(GNOME_USERMENU, GNOME_USERMENU._onLockScreenActivate));
        this.addMenuItem(this._itemLockScreen);
        this._itemSwitchUser = new PanelTools.PopupMenuItem(_("Switch User"));
        this._itemSwitchUser.connect('activate', Lang.bind(GNOME_USERMENU, GNOME_USERMENU._onLoginScreenActivate));
        this.addMenuItem(this._itemSwitchUser);
        this._itemLogout = new PanelTools.PopupMenuItem(_("Log Out..."));
        this._itemLogout.connect('activate', Lang.bind(GNOME_USERMENU, GNOME_USERMENU._onQuitSessionActivate));
        this.addMenuItem(this._itemLogout);
        this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemGNOMEShellRestart = new PanelTools.PopupMenuItem(_("GNOME Shell restart"));
        this._itemGNOMEShellRestart.connect('activate', Lang.bind(GNOME_USERMENU, _onGNOMEShellRestartActivate));
        this.addMenuItem(this._itemGNOMEShellRestart);
        this._itemTerminal = new PanelTools.PopupMenuItem(_("Terminal"));
        this._itemTerminal.connect('activate', Lang.bind(GNOME_USERMENU, _onTerminalActivate));
        this.addMenuItem(this._itemTerminal);
        this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemSuspend = new PanelTools.PopupMenuItem(_("Suspend"));
        this._itemSuspend.connect('activate', Lang.bind(GNOME_USERMENU, _onSuspendActivate));
        this.addMenuItem(this._itemSuspend);
        this._itemHibernate = new PanelTools.PopupMenuItem(_("Hibernate"));
        this._itemHibernate.connect('activate', Lang.bind(GNOME_USERMENU, _onHibernateActivate));
        this.addMenuItem(this._itemHibernate);
        this._itemPowerOff = new PanelTools.PopupMenuItem(_("Power Off..."));
        this._itemPowerOff.actor.add_style_pseudo_class('alternate');
        this._itemPowerOff.connect('activate', Lang.bind(GNOME_USERMENU, _onPowerOffActivate));
        this.addMenuItem(this._itemPowerOff);
        this._connectMenu = GNOME_USERMENU.menu.connect('open-state-changed', Lang.bind(this, function(menu, open) {
            if (!open) return;
            _updateLockScreen.call(GNOME_USERMENU, this._itemLockScreen.actor);
            _updateSwitchUser.call(GNOME_USERMENU, this._itemSwitchUser);
            _updateLogout.call(GNOME_USERMENU, this._itemLogout.actor);
            this._itemTerminal.visible = GLib.find_program_in_path('gnome-terminal');
            _updateSuspend.call(GNOME_USERMENU, this._itemSuspend.actor);
            _updateHibernate.call(GNOME_USERMENU, this._itemHibernate.actor);
            _updatePowerOff.call(GNOME_USERMENU, this._itemPowerOff.actor);
        }));
        this.connect('destroy', Lang.bind(this, function() {
            GNOME_USERMENU.menu.disconnect(this._connectMenu);
        }));
    }
});

const PopupUserMenuAlternateMenuSection = new Lang.Class({
    Name: 'PopupUserMenuAlternateMenuSection',
    Extends: PopupMenu.PopupMenuSection,

    _init: function() {
        this.parent();
        this.actor.add_style_class_name('panelmanager-user-menu');
        this.addMenuItem(new PopupIMStatusChooserMenuItem(this.icon));
        this.addMenuItem(new PopupNotificationsSwitchMenuItem());
        this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemCommunication = new PanelTools.PopupSubMenuMenuItem(_("Communication Applications"), true, { settings: false });
        this._itemCommunication.menu.addMenuItem(new PopupCommunicationApplicationsMenuSection(false, false));
        this._itemCommunication.setIconName('user-invisible-symbolic');
        this.addMenuItem(this._itemCommunication);
        this._itemBasicSettings = new PanelTools.PopupSubMenuMenuItem(_("BasicSettings"), true, { settings: false });
        this._itemBasicSettings.menu.addMenuItem(new PopupBasicSettingsMenuSection(false));
        this._itemBasicSettings.setIconName('preferences-system-symbolic');
        this.addMenuItem(this._itemBasicSettings);
        this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemUserSession = new PanelTools.PopupSubMenuMenuItem('', true, { settings: false });
        this._itemUserSession.icon.style_class = 'system-switch-user-submenu-icon';
        this._itemLockScreen = new PanelTools.PopupMenuItem(_("Lock Screen"));
        this._itemLockScreen.connect('activate', Lang.bind(GNOME_USERMENU, GNOME_USERMENU._onLockScreenActivate));
        this._itemUserSession.menu.addMenuItem(this._itemLockScreen);
        this._itemSwitchUser = new PanelTools.PopupMenuItem(_("Switch User"));
        this._itemSwitchUser.connect('activate', Lang.bind(GNOME_USERMENU, GNOME_USERMENU._onLoginScreenActivate));
        this._itemUserSession.menu.addMenuItem(this._itemSwitchUser);
        this._itemLogout = new PanelTools.PopupMenuItem(_("Log Out..."));
        this._itemLogout.connect('activate', Lang.bind(GNOME_USERMENU, GNOME_USERMENU._onQuitSessionActivate));
        this._itemUserSession.menu.addMenuItem(this._itemLogout);
        this.addMenuItem(this._itemUserSession);
        this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemActions = new PanelTools.PopupActionMenuItem();
        this._actionPanelManagerSettings = this._itemActions.addActionButton('preferences-other-symbolic', 'Settings for PanelManager');
        this._actionPanelManagerSettings.connect('clicked', Lang.bind(GNOME_USERMENU, _onPanelManagerSettingsActivate));
        this._actionGNOMEShellRestart = this._itemActions.addActionButton('view-refresh-symbolic', _("GNOME Shell restart"));
        this._actionGNOMEShellRestart.connect('clicked', Lang.bind(GNOME_USERMENU, _onGNOMEShellRestartActivate));
        this._actionTerminal = this._itemActions.addActionButton('utilities-terminal-symbolic', _("Terminal"));
        this._actionTerminal.connect('clicked', Lang.bind(GNOME_USERMENU, _onTerminalActivate));
        this._actionSuspend = this._itemActions.addActionButton('media-playback-pause-symbolic', _("Suspend"));
        this._actionSuspend.connect('clicked', Lang.bind(GNOME_USERMENU, _onSuspendActivate));
        this._actionHibernate = this._itemActions.addActionButton('media-playback-stop-symbolic', _("Hibernate"));
        this._actionHibernate.connect('clicked', Lang.bind(GNOME_USERMENU, _onHibernateActivate));
        this._actionPowerOff = this._itemActions.addActionButton('system-shutdown-symbolic', _("Power Off..."));
        this._actionPowerOff.connect('clicked', Lang.bind(GNOME_USERMENU, _onPowerOffActivate));
        this.addMenuItem(this._itemActions);
        this._connectMenu = GNOME_USERMENU.menu.connect('open-state-changed', Lang.bind(this, function(menu, open) {
            if (!open) return;
            this._updateUserName();
            _updateLockScreen.call(GNOME_USERMENU, this._itemLockScreen.actor);
            _updateSwitchUser.call(GNOME_USERMENU, this._itemSwitchUser);
            _updateLogout.call(GNOME_USERMENU, this._itemLogout.actor);
            this._actionTerminal.visible = GLib.find_program_in_path('gnome-terminal');
            _updateSuspend.call(GNOME_USERMENU, this._actionSuspend);
            _updateHibernate.call(GNOME_USERMENU, this._actionHibernate);
            _updatePowerOff.call(GNOME_USERMENU, this._actionPowerOff);
        }));
        this.connect('destroy', Lang.bind(this, function() {
            GNOME_USERMENU.menu.disconnect(this._connectMenu);
        }));
    },

    _updateUserName: function () {
        this._itemUserSession.label.set_text(GNOME_USERMENU._user.is_loaded ? GNOME_USERMENU._user.get_real_name() : '');
        let iconFile = GNOME_USERMENU._user.get_icon_file();
        if (iconFile && GLib.file_test(iconFile, GLib.FileTest.EXISTS)) {
            let file = Gio.File.new_for_path(iconFile);
            let gicon = new Gio.FileIcon({ file: file });
            this._itemUserSession.icon.gicon = gicon;
            this._itemUserSession.icon.add_style_class_name('user-icon');
            this._itemUserSession.icon.remove_style_class_name('default-icon');
        } else {
            this._itemUserSession.icon.icon_name = 'avatar-default-symbolic';
            this._itemUserSession.icon.add_style_class_name('default-icon');
            this._itemUserSession.icon.remove_style_class_name('user-icon');
        }
    }
});

const PopupUserMenuDefaultMenuSection = new Lang.Class({
    Name: 'PopupUserMenuDefaultMenuSection',
    Extends: PopupMenu.PopupMenuSection,

    _init: function() {
        this.parent();
        let menuItems = GNOME_USERMENU.menu._getMenuItems();
        for (let i = 0; i < menuItems.length; i++) {
            GNOME_USERMENU.menu.box.remove_actor(menuItems[i].actor);
            this.addMenuItem(menuItems[i]);
        }
        this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemPanelManagersettings = new PanelTools.PopupMenuItem(_("Settings for PanelManager"));
        this._itemPanelManagersettings.connect('activate', Lang.bind(GNOME_USERMENU, _onPanelManagerSettingsActivate));
        this.addMenuItem(this._itemPanelManagersettings);
        this.actor.connect('destroy', Lang.bind(this, function() {
            this._itemPanelManagersettings.destroy();
        }));
    }
});

const BasicSettingsButton = new Lang.Class({
    Name: 'PanelManager.BasicSettings',
    Extends: PanelTools.Button,

    _init: function() {
        this.parent(0.0, _("BasicSettings"), { icon_name: 'preferences-system-symbolic' });
        this.menu.addMenuItem(new PopupBasicSettingsMenuSection(true));
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        let item = new PanelTools.PopupMenuItem(_("Settings for PanelManager"));
        item.connect('activate', Lang.bind(GNOME_USERMENU, _onPanelManagerSettingsActivate));
        this.menu.addMenuItem(item);
    }
});

const CommunicationButton = new Lang.Class({
    Name: 'PanelManager.Communication',
    Extends: PanelTools.Button,

    _init: function() {
        this.parent(0.0, _("Communication"), { icon_name: 'user-invisible-symbolic' });
        this.menu.addMenuItem(new PopupIMStatusChooserMenuItem(this.icon));
        this.menu.addMenuItem(new PopupNotificationsSwitchMenuItem());
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this.menu.addMenuItem(new PopupCommunicationApplicationsMenuSection(true, true));
    }
});

const UserMenuButton = new Lang.Class({
    Name: 'UserMenuButton',

    _init: function() {
        this.Communication = PanelTools.addToStatusArea('dschaerfCommunication', new CommunicationButton(), PanelTools.getNumChildren('right') - 1, 'right');
        this.BasicSettings = PanelTools.addToStatusArea('dschaerfBasicSettings', new BasicSettingsButton(), PanelTools.getNumChildren('right') - 1, 'right');
        let box = GNOME_USERMENU.actor.get_children()[0];
        this._iconSettings = new St.Icon({ icon_name: 'preferences-system-symbolic', style_class: 'popup-menu-icon' });
        box.add(this._iconSettings, { y_align: St.Align.MIDDLE, y_fill: false });
        this._iconPower = new St.Icon({ icon_name: 'system-shutdown-symbolic', style_class: 'popup-menu-icon' });
        box.add(this._iconPower, { y_align: St.Align.MIDDLE, y_fill: false });
        this._itemUserMenuDefault = new PopupUserMenuDefaultMenuSection();
        GNOME_USERMENU.menu.addMenuItem(this._itemUserMenuDefault);
        this._itemUserMenuAdvanced = new PopupUserMenuAdvancedMenuSection();
        GNOME_USERMENU.menu.addMenuItem(this._itemUserMenuAdvanced);
        this._itemUserMenuAlternate = new PopupUserMenuAlternateMenuSection();
        GNOME_USERMENU.menu.addMenuItem(this._itemUserMenuAlternate);
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = [
            this._settings.connect('changed::'+KEY_SHOW_STATUSAREA, Lang.bind(this, this._onShowStatusArea)),
            this._settings.connect('changed::'+KEY_SHOW_STATUSMENU, Lang.bind(this, this._onShowStatusMenu))
        ];
        this._onShowStatusMenu();
    },

    _onShowStatusArea: function() {
        this._updateVisibility(this._settings.get_enum(KEY_SHOW_STATUSAREA), null);
    },

    _onShowStatusMenu: function() {
        this._updateVisibility(null, this._settings.get_enum(KEY_SHOW_STATUSMENU));
    },

    _updateVisibility: function(showStatusArea, showStatusMenu) {
        if (showStatusArea == null) showStatusArea = this._settings.get_enum(KEY_SHOW_STATUSAREA);
        if (showStatusMenu == null) showStatusMenu = this._settings.get_enum(KEY_SHOW_STATUSMENU);
        this.BasicSettings.actor.visible = showStatusArea == STATUSAREA_ADVANCED;
        this.Communication.actor.visible = showStatusArea == STATUSAREA_ADVANCED;
        GNOME_USERMENU._iconBox.visible = showStatusArea != STATUSAREA_ADVANCED && showStatusMenu != STATUSMENU_ICON;
        GNOME_USERMENU._name.visible = showStatusArea == STATUSAREA_DEFAULT && showStatusMenu != STATUSMENU_ICON;
        this._iconSettings.visible = showStatusArea == STATUSAREA_ALTERNATE && showStatusMenu != STATUSMENU_ICON;
        this._iconPower.visible = showStatusArea != STATUSAREA_DEFAULT || showStatusMenu == STATUSMENU_ICON;
        this._itemUserMenuDefault.actor.visible = showStatusArea == STATUSAREA_DEFAULT;
        this._itemUserMenuAdvanced.actor.visible = showStatusArea == STATUSAREA_ADVANCED;
        this._itemUserMenuAlternate.actor.visible = showStatusArea == STATUSAREA_ALTERNATE;
    },

    destroy: function() {
        for each(let connection in this._connectSettings) {
            this._settings.disconnect(connection);
        }
        this._updateVisibility(STATUSAREA_DEFAULT, STATUSMENU_DEFAULT);
        this._itemUserMenuAlternate.destroy();
        this._itemUserMenuAdvanced.destroy();
        let menuItems = this._itemUserMenuDefault._getMenuItems();
        for (let i = 0; i < menuItems.length; i++) {
            this._itemUserMenuDefault.box.remove_actor(menuItems[i].actor);
            GNOME_USERMENU.menu.addMenuItem(menuItems[i]);
        }
        GNOME_USERMENU._updateSuspendOrPowerOff();
        GNOME_USERMENU._updateSwitchUser();
        GNOME_USERMENU._updateLogout();
        GNOME_USERMENU._updateLockScreen();
        this._itemUserMenuDefault.destroy();
        this._iconPower.destroy();
        this._iconSettings.destroy();
        this.BasicSettings.destroy();
        this.Communication.destroy();
    }
});

function getExtension() {
    return GNOME_USERMENU ? new UserMenuButton() : null;
}
