// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*
  Copyright (c) 2011-2012, Giovanni Campagna <scampa.giovanni@gmail.com>
                2012-2017, Dieter Schärf <ds@dschaerf.de>

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the GNOME nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

const Gio = imports.gi.Gio;

const Config = imports.misc.config;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const CURRENT_GETTEXT_DOMAIN = Me.metadata['gettext-domain'];
const CURRENT_SETTINGS_SCHEMA = Me.metadata['settings-schema'];

const GNOME_SHELL_VERSION = Config.PACKAGE_VERSION.split('.');

const Gettext = imports.gettext;
const _ = Gettext.domain(CURRENT_GETTEXT_DOMAIN).gettext;

/**
 * checkGSV:
 * @major:                GNOME Shell main version number
 * @minor:                GNOME Shell version number
 * @revision: (optional): GNOME Shell revision number
 *
 * Checks whether or not the current GNOME Shell version matches the
 * version you are looking for.
 */
function checkGSV(major, minor, revision) {
    if (GNOME_SHELL_VERSION[0] == major.toString() && GNOME_SHELL_VERSION[1] == minor.toString()) {
        if (!revision || GNOME_SHELL_VERSION[2] == revision.toString()) return true;
    }
    return false;
}

/**
 * getSettings:
 * @schema:          (optional): the GSettings schema id
 * @dontCheckSchema: (optional): boolean value for schema check
 *
 * Builds and return a GSettings schema for @schema, using schema files
 * in extensionsdir/schemas. If @schema is not provided, it is taken from
 * metadata['settings-schema'].
 *
 * Is @schema not found and the value of @dontCheckSchema is TRUE, then
 * the return value of schema is NULL.
 */
function getSettings(schema, dontCheckSchema) {
    schema = schema || CURRENT_SETTINGS_SCHEMA;
    const GioSSS = Gio.SettingsSchemaSource;
    // check if this extension was built with "make zip-file", and thus
    // has the schema files in a subfolder
    // otherwise assume that extension has been installed in the
    // same prefix as gnome-shell (and therefore schemas are available
    // in the standard folders)
    let schemaDir = Me.dir.get_child('schemas');
    let schemaSource;
    if (schemaDir.query_exists(null)) {
        schemaSource = GioSSS.new_from_directory(schemaDir.get_path(), GioSSS.get_default(), false);
    } else {
        schemaSource = GioSSS.get_default();
    }
    let schemaObj = schemaSource.lookup(schema, true);
    if (!schemaObj) {
        if (dontCheckSchema) return null;
        throw new Error(_("Schema '%s' could not be found for extension '%s'. Please check your installation.").format(schema, Me.metadata.uuid));
    }
    return new Gio.Settings({ settings_schema: schemaObj });
}

/**
 * getSorted:
 * @a: first string
 * @b: second string
 *
 * Compares @a and @b and returns the result:
 * result  0: @a equal @b
 * result  1: @a greater @b
 * result -1: @a lesser @b
 */
function getSorted(a, b) {
    if (a.toUpperCase() == b.toUpperCase()) return 0;
    if (a.toUpperCase() > b.toUpperCase()) return 1;
    return -1;
}

/**
 * initTranslations:
 * @domain: (optional): the gettext domain to use
 *
 * Initialize Gettext to load translations from extensionsdir/locale.
 * If @domain is not provided, it will be taken from metadata['gettext-domain']
 */
function initTranslations(domain) {
    domain = domain || CURRENT_GETTEXT_DOMAIN;
    // check if this extension was built with "make zip-file", and thus
    // has the locale files in a subfolder
    // otherwise assume that extension has been installed in the
    // same prefix as gnome-shell
    let localeDir = Me.dir.get_child('locale');
    if (localeDir.query_exists(null)) {
        Gettext.bindtextdomain(domain, localeDir.get_path());
    } else {
        Gettext.bindtextdomain(domain, Config.LOCALEDIR);
    }
}
